{
open Parser
let sym_table = [("true",Parser.BOOL(true)); ("false", Parser.BOOL(false));
		("fst", Parser.PRIM("fst"));("snd", Parser.PRIM("snd"))];;
}

let space = [' ' '\t' '\n' '\r']
let digit = ['0'-'9']
let lower = ['a'-'z']
let upper = ['A'-'Z']

rule token = parse
| space+
    { token lexbuf }
| "/*"
    { comment lexbuf;
      token lexbuf }
| "?"
    { INPUT(Lexing.lexeme_start lexbuf) }
| "!"
    { OUTPUT(Lexing.lexeme_start lexbuf) }
| "."
    { PERIOD}
| "O"
    { ZERO}
| "*" 
    { REP}
| ":" 
    {COLON}
| "+"
    { PLUS }
| "-"
    { MINUS }
| "/"
    { DIV }
| "|"
    { PAR }
| "(#T" {SLPAREN}
| "#)"  {SRPAREN}
| "("
    { LPAREN }
| ")"
    { RPAREN }
| ","
    { COMMA }
| "new"
    { NU }
| "in"
    { IN }
| "let"
    { LET}
| "if"
    {IF}
| "then"
    {THEN}
| "else"
    {ELSE}
| "="
    {EQ}
| ">"
    {GT}
| "<"
    {LT}
| ">="
    {GTEQ}
| "<="
    {LTEQ}
| "[]"
    {NIL}
| "nil"
    {NIL}
| "null"
    {PRIM("null")}
| "cons"
    {PRIM("cons")}
| "hd"
    {PRIM("hd")}
| "tl"
    {PRIM("tl")}
| digit+
    { INT(int_of_string (Lexing.lexeme lexbuf)) }
| lower (digit|lower|upper|'_')*
    { let s = Lexing.lexeme lexbuf in
      try List.assoc s sym_table with Not_found -> IDENT(s)}
| eof
    { EOF }
| _
    { Format.eprintf "unknown token %s near characters %d-%d@."
	(Lexing.lexeme lexbuf)
	(Lexing.lexeme_start lexbuf)
	(Lexing.lexeme_end lexbuf);
      failwith "lex error" }
and comment = parse
| "*/"
    { () }
| "/*"
    { comment lexbuf;
      comment lexbuf }
| eof
    { failwith "unterminated comment" }
| _
    { comment lexbuf }
