
let debug = ref false;;

(*** copy a process expression from the input stream to the output stream,
     while annotating each input and output with the analysis result 
***)
let copy_and_insert in_strm out_strm result =
  let copy_one_char () =
     let c = input_char in_strm in
       if !Flag.html then
         if c='!' then output_string out_strm ("<font color=\"red\"><strong>!</strong></font>")
         else if c='?' then output_string out_strm ("<font color=\"red\"><strong>?</strong></font>")
         else output_char out_strm c
       else output_char out_strm c
  in
  let write_lockfree_output n = 
   if !debug then  output_string out_strm  ("!["^(string_of_int(n))^"]")
   else if !Flag.html then
           output_string out_strm  "<font color=\"blue\"><strong>!</strong></font>"
        else output_string out_strm  "!!" 
  in
  let write_lockfree_input n = 
   if !debug then  output_string out_strm  ("?["^(string_of_int(n))^"]")
   else if !Flag.html then
           output_string out_strm  "<font color=\"blue\"><strong>?</strong></font>"
        else output_string out_strm  "??" 
  in
  let rec copy ()=
     (copy_one_char (); copy()) 
  in
  let write_begin_of_html() = 
     output_string out_strm "\n\n"
  in
  let write_end_of_html() = 
     output_string out_strm "\n\n"
  in
  let rec copy_and_insert_aux result pos =
   match result with
    [] -> copy()
  | (pos',level)::result' ->
      if pos<pos' then 
         (copy_one_char(); copy_and_insert_aux result (pos+1))
      else 
        if pos=pos' then
          match level with
              Iflow.LevConst n -> 
                    let c = input_char in_strm in
                    let _ = if c='!' then write_lockfree_output n
                            else if c='?' then write_lockfree_input n
                            else raise (Iflow.SomethingIsWrongIn "copy_and_insert:1") in
                        copy_and_insert_aux result' (pos+1)
            | _ -> copy_and_insert_aux result' pos
        else raise (Iflow.SomethingIsWrongIn "copy_and_insert:2")
  in
     (if !Flag.html then write_begin_of_html() else ();
     try (copy_and_insert_aux result 0)
     with End_of_file -> 
       (close_in in_strm; 
        if !Flag.html then write_end_of_html() else ();
        flush out_strm));;

let analyze filename p = 
     let p' = Syntax.alphaconv (Syntax.normalize_procexp p) in
     let result = Iflow.lockfreedom_analysis p' in 
     let in_strm = 
         try open_in filename
         with Sys_error _ -> 
            (print_string ("Cannot open file: "^filename^"\n");
             exit(-1)) in
     let out_strm = stdout in
        copy_and_insert in_strm out_strm result;;

let analyze_wl filename p = 
  try
     let _ = (Flag.wlock := true) in
     let p' = Syntax.alphaconv (Syntax.normalize_procexp p) in
     let result = Iflow.wlockfreedom_analysis p' in 
     let in_strm = 
         try open_in filename
         with Sys_error _ -> 
            (print_string ("Cannot open file: "^filename^"\n");
             exit(-1)) in
     let out_strm = stdout in
        copy_and_insert in_strm out_strm result
  with
     Iflow.FailWlockAnalysis -> print_string "Analysis Failed\n";;

let analyze_wlauto filename p = 
   let _ = Flag.wlauto := true in
   let p' = Syntax.alphaconv (Syntax.normalize_procexp p) in
   let result = Iflow.wlockfreedom_auto_analysis p' in 
   let in_strm = 
         try open_in filename
         with Sys_error _ -> 
            (print_string ("Cannot open file: "^filename^"\n");
             exit(-1)) in
     let out_strm = stdout in
        copy_and_insert in_strm out_strm result

let analyze_d filename p =
  let _ = (Flag.livelock := false) in
  let _ = analyze filename p in
   (Flag.livelock := false);;

let analyze_u (p:unit Syntax.procexp) = 
     let result = Iflow.useless_code_elimination p in
        (Syntax.pp_proc None result; print_string "\n"; flush stdout);;

let analyze_s p = 
     let (p', env) = Stype.elaborate p in
        (Syntax.pp_proc (Some Stype.string_of_type) p'; 
         print_string "\n where its type environment is:\n"; 
	 Stype.print_env env;
         flush stdout);;

let analyze_i (p:unit Syntax.procexp) = 
     let result = Iflow.iflow_analysis p in 
        (Syntax.pp_proc None result; print_string "\n"; flush stdout);;

let analyze_t (p:unit Syntax.procexp) = 
     try let _ = Termination.elaborate p in ()
     with Termination.Fatal s -> print_string ("Fatal: "^s^"\n")
        | Termination.Error s -> print_string (s^"\n")

let parse file = 
  let _ = Iflow.reset_names() in 
  let in_strm = open_in file in
  let lexbuf = Lexing.from_channel in_strm in
  let p = Syntax.normalize_procexp(Parser.main Lexer.token lexbuf) in
  let _ = close_in in_strm in
     p;;

let main () =
  let _ = print_string "TyPiCal 1.6.2: A Type-based static analyzer for the Pi-Calculus\n" in
  let (analysis_func, filename) =
    try 
      let index =
        if Sys.argv.(1) = "-h" then 
         (** if -h option is specified, output the result in html format **)
          (Flag.html:= true; 2)
        else (Flag.html:= false; 1)
      in
       match Sys.argv.(index) with
         "-d" -> (analyze Sys.argv.(index+1), (Flag.livelock:=false; Sys.argv.(index+1)))
       | "-l" -> (analyze Sys.argv.(index+1), (Flag.livelock:=true; Sys.argv.(index+1)))
       | "-wl" -> (analyze_wl Sys.argv.(index+1), (Flag.livelock:=true; Sys.argv.(index+1)))
       | "-wlauto" -> (analyze_wlauto Sys.argv.(index+1), (Flag.livelock:=true; Sys.argv.(index+1)))
       | "-u" -> (analyze_u, (Flag.livelock:=true; Sys.argv.(index+1)))
       | "-s" -> (analyze_s, (Flag.livelock:=false; Sys.argv.(index+1)))
       | "-i" -> (analyze_i, (Flag.livelock:=true; Sys.argv.(index+1)))
       | "-t" -> (analyze_t, (Flag.livelock:=false; Sys.argv.(index+1)))
       | _ -> (analyze Sys.argv.(index), Sys.argv.(index))
    with 
       Invalid_argument _ -> 
         (print_string "Usage:\n";
          print_string "  typical [option] filename\n";
          print_string "  Available options are:\n";
          print_string "    -d: deadlock-freedom analysis\n";
          print_string "    -i: information flow analysis\n";
          print_string "    -l: lock-freedom analysis (default)\n";
          print_string "    -wl: weak lock-freedom analysis\n";
          print_string "    -wlauto: weak lock-freedom analysis (with termination annotation inference)\n";
          print_string "    -s: simple type inference\n";
          print_string "    -u: useless code elimination\n";
          print_string "    -t: termination analysis\n";
          exit(-1)
         ) in
   let _ = Flag.start_t := Sys.time() in
   let in_strm = 
         try open_in filename 
         with Sys_error _ -> 
            (print_string ("Cannot open file: "^filename^"\n");
             exit(-1)) in
   let _ = if !Flag.html then print_string "<pre>\n" else () in
   let _ = print_string ("analyzing "^filename^"...\n") in
   let lexbuf = Lexing.from_channel in_strm in
   let p = try
              Parser.main Lexer.token lexbuf
           with 
              Failure _ -> exit(-1) (*** exception raised by the lexicl analyer ***)
            | Parsing.Parse_error -> 
               (print_string "Parse error\n";
                exit(-1)) in
   let _ = 
         try close_in in_strm
         with Sys_error _ -> 
            (print_string ("Cannot close file: "^filename^"\n");
             exit(-1)) in
   let _ = Iflow.reset_names() in 
   let _ = try
             analysis_func p 
           with 
              Stype.Unif _ -> 
                (print_string "The process is ill-typed.\n"; exit(-1))
            | Iflow.Failure_of_Weak_Unification _ ->
                (print_string "The process is ill-typed.\n"; exit(-1))
            | Scc.Cycle ->
                (print_string "The process is ill-typed.\n"; exit(-1)) 
            | Multisetorder.SCT_NOTFOUND ->
                (print_string "Install sct\n"; exit(-1))
            | _ -> 
                (print_string "Fatal error: send a bug report\n"; exit(-1)) in
   let end_t = Sys.time() in
       (print_string ("Elapsed Time: "^(string_of_float (end_t -. !Flag.start_t))^"sec\n");
        if !Flag.html then print_string "</pre>\n" else ());;

if !Sys.interactive then
  ()
else
  main();;


  
