type vid = string
type cid = string
type slevanot = Hi | Low
type primname = string
type pc = int
type 'a valexp = (*** value expression ***)
             Var of vid   (*** variable ***)
           | Unit          (*** unit ***)
           | SBool of bool * slevanot (*** boolean annotated with its security level ***)
           | SInt of int * slevanot   (*** integer annotated with its security level ***)
           | Bool of bool             (*** boolean ***)
           | Int of int               (*** integer ***)
           | Prim of primname * 'a option      (*** primitive ***)
           | App of 'a valexp * 'a valexp * pc (*** application ***)
           | Pair of 'a valexp * 'a valexp;;   (*** pair ***)
type 'a procexp = Zero
          | Out of vid * 'a valexp * 'a procexp * pc  
          | In of vid * vid * 'a * 'a procexp * pc
          | Par of 'a procexp * 'a procexp
          | Rep of 'a procexp
          | Nu of vid * 'a * 'a procexp * pc
          | If of 'a valexp * 'a procexp * 'a procexp * pc
          | MatchL of vid * 'a matchlist * pc (*** case analysis on data structures ***)
          | Let of vid * 'a * 'a valexp * 'a procexp * pc
          | Special of 'a procexp
and 'a matchlist = (cid * 'a procexp) list;;
val normalize_procexp: 'a procexp -> 'a procexp
val reduce_proc: 'a procexp -> 'a procexp
val print_proc: int -> ('a -> string) option -> 'a procexp -> unit
val pp_proc: ('a -> string) option ->'a procexp ->  unit
val make_vsubst: vid -> 'a valexp -> (vid,'a valexp) Utilities.substitution
val apply_vsubst_v: (vid,'a valexp) Utilities.substitution -> 'a valexp -> 'a valexp
val comp_vsubst: (vid, 'a valexp) Utilities.substitution -> 
                  (vid, 'a valexp) Utilities.substitution -> (vid, 'a valexp) Utilities.substitution
val map_for_t: ('a -> 'b) -> 'a procexp -> 'b procexp
val alphaconv: 'a procexp -> 'a procexp
val undo_alphaconv: 'a procexp -> 'a procexp
val fv_proc: 'a procexp -> vid list
val sizeof: 'a procexp -> int
val get_subproc: 'a procexp -> pc -> 'a procexp
val rep_restricted: 'a procexp -> bool
val add_annotation: 'a procexp -> pc -> 'a procexp
val pcs: 'a procexp -> pc list  (*** (pcs p) returns the set of pc's occuring in p ***)
